﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	/// <summary>
	/// Hrací plocha, ve které jsou spadlé kostky
	/// </summary>
	public class HraciPlocha
    {
        private int[,] policka;
        /// <summary>
        /// Šířka hrací plochy v políčkách
        /// </summary>
        private int sirka;
        /// <summary>
        /// Výška hrací plochy v políčkách
        /// </summary>
        private int vyska;
        /// <summary>
        /// Vykreslovací souřadnice hrací plochy
        /// </summary>
        private Vector2 pozice;

        /// <summary>
        /// Vytvoří novou herní plochu
        /// </summary>
        /// <param name="sirka">Šířka herní plochy v kostkách</param>
        /// <param name="vyska">Výška herní plochy v kostkách</param>
        /// <param name="pozice">Vykreslovací souřadnice hrací plochy</param>
        public HraciPlocha(int sirka, int vyska, Vector2 pozice)
        {
            this.sirka = sirka;
            this.vyska = vyska;
            this.pozice = pozice;
            Vyprazdni();
        }

        /// <summary>
        /// Posadí kostku nahoru, do středu hrací plochy
        /// </summary>
        /// <param name="kostka"></param>
        /// <returns></returns>
        public Vector2 PosadKostku(Kostka kostka)
        {
            return new Vector2((sirka / 2) - (kostka.hrana / 2), 0);
        }

        /// <summary>
        /// Funkce vrátí true, když kostka, posunutá na danou pozici, koliduje s jinou kostkou v hrací ploše
        /// </summary>
        /// <param name="kostka">Kostka</param>
        /// <param name="pozice">Pozice kostky</param>
        /// <returns>True při kolizi, jinak false</returns>
        public bool Kolize(Kostka kostka, Vector2 pozice)
        {
            // budeme předpokládat, že nedošlo ke kolizi
            bool kolize = false;
            // projetí všech políček kostky
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                    // Pokud je políčko v kostce plné a
                    if ((kostka.Policka[i, j] > 0) &&
                        // vyjelo z hrací plochy nebo
                       ((j + pozice.Y >= vyska) || (i + pozice.X < 0) || (i + pozice.X >= sirka) ||
                        // překrývá se s jiným políčkem hrací plochy
                       (policka[Convert.ToInt32(i + pozice.X), Convert.ToInt32(j + pozice.Y)] > 0)))
                        // došlo ke kolizi
                        kolize = true;
            return kolize;
        }

        /// <summary>
        /// Připojí kostku do políček hrací plochy, tedy mezi spadlé kostky
        /// </summary>
        /// <param name="kostka">Kostka</param>
        public void Pripoj(Kostka kostka)
        {
            // projetí všech políček kostky
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                {
                    // získání souřadnic políčka v hrací ploše
                    int x = Convert.ToInt32(i + kostka.pozice.X);
                    int y = Convert.ToInt32((j + kostka.pozice.Y));
                    // políčko je zaplněné a nevyčnívá zhora z hrací plochy
                    if ((kostka.Policka[i, j] > 0) && (y >= 0))
                        policka[x, y] = kostka.Policka[i, j];
                }
        }

        /// <summary>
        /// Najde plné řady políček a vymaže je
        /// </summary>
        /// <returns>Počet vymazaných řad</returns>
        public int VymazRady()
        {
            // kontrola plných řad
            int pocet = 0;
            for (int rada = 0; rada < vyska; rada++)
            {
                // předpokládáme, že je řada plná
                bool plna = true;
                // pokud je někde prázdné políčko, plná není
                for (int i = 0; i <= (sirka - 1); i++)
                    if (policka[i, rada] == 0)
                        plna = false;
                // smazání plné řady
                if (plna)
                {
                    // posun políček nad řadou dolů
                    for (int j = (rada - 1); j > 0; j--)
                        for (int i = 0; i < sirka; i++)
                            policka[i, j + 1] = policka[i, j];
                    pocet++;
                }
            }
            return pocet;
        }

        /// <summary>
        /// Vyprázdni hrací plochu
        /// </summary>
        public void Vyprazdni()
        {
            policka = new int[sirka, vyska];
        }

        /// <summary>
        /// Vrátí pozici zaměřovače, tedy místo, kam kostka dopadne
        /// </summary>
        /// <param name="kostka">Aktuální kostka</param>
        /// <returns>Pozice zaměřovače</returns>
        public Vector2 SpocitejSouradniceZamerovace(Kostka kostka)
        {
            Vector2 pozice = kostka.pozice;
            // zkusíme posouvat kostku dolů tak dlouho, dokud nebude kolidovat
            while (!Kolize(kostka, pozice))
                pozice.Y++;
            // vrátíme souřadnici o 1 nahoru, kdy ještě nekolidovala
            return new Vector2(kostka.pozice.X, pozice.Y - 1);
        }

        /// <summary>
        /// Vykreslí políčka v hrací ploše
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch</param>
        /// <param name="sprity">Pole spritů pro políčka</param>
        public void Vykresli(SpriteBatch spriteBatch, Texture2D[] sprity)
        {
            for (int j = 0; j <= (vyska - 1); j++)
                for (int i = 0; i <= (sirka - 1); i++)
                    if (policka[i, j] != 0)
                        spriteBatch.Draw(sprity[policka[i, j] - 1], new Vector2(pozice.X + i * sprity[0].Width, pozice.Y + j * sprity[0].Height), Color.White);
        }

    }
}
