﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Robotris
{
	/*
    *  _____ _______         _                      _
    * |_   _|__   __|       | |                    | |
    *   | |    | |_ __   ___| |___      _____  _ __| | __  ___ ____
    *   | |    | | '_ \ / _ \ __\ \ /\ / / _ \| '__| |/ / / __|_  /
    *  _| |_   | | | | |  __/ |_ \ V  V / (_) | |  |   < | (__ / /
    * |_____|  |_|_| |_|\___|\__| \_/\_/ \___/|_|  |_|\_(_)___/___|
    *                   ___
    *                  |  _|___ ___ ___
    *                  |  _|  _| -_| -_|  LICENCE
    *                  |_| |_| |___|___|
    *
    * IT ZPRAVODAJSTVÍ  <>  PROGRAMOVÁNÍ  <>  HW A SW  <>  KOMUNITA
    *
    * Tento zdrojový kód pochází z IT sociální sítě WWW.ITNETWORK.CZ
    *
    * Můžete ho upravovat a používat jak chcete, musíte však zmínit
    * odkaz na http://www.itnetwork.cz
    */

	public class Kostka
    {
        /// <summary>
        /// Jednotlivá políčka kostky
        /// </summary>
        public int[,] Policka { get; private set; }
        /// <summary>
        /// Pozice kostky v hrací ploše
        /// </summary>
        public Vector2 pozice;
        /// <summary>
        /// Generátor náhodných čísel
        /// </summary>
        private Random nahodnyGenerator;
        /// <summary>
        /// Hrana kostky
        /// </summary>
        public int hrana;

        /// <summary>
        /// Zkopíruje políčka z jiných políček, vyhýbáme se tak předání referencí
        /// </summary>
        /// <param name="policka">Vzorová políčka</param>
        /// <returns>Kopie vzorových políček</returns>
        private int[,] ZkopirujPolicka(int[,] policka)
        {
            int[,] nova = new int[4, 4];
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                    nova[i, j] = policka[i, j];
            return nova;
        }

        /// <summary>
        /// Vygeneruje náhodné indexy pro sprity a dosadí je do plných políček
        /// </summary>
        private void GenerujSprity()
        {
            // výběr políček pro kostku
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                    if (Policka[i, j] > 0)
                        Policka[i, j] = nahodnyGenerator.Next(1, 16);
        }

        /// <summary>
        /// Spočítá hranu kostky
        /// </summary>
        private void ZjistiHranu()
        {
            hrana = 1;
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                {
                    // nalezeno políčko dále od levého horního rohu, než je současná hrana
                    if ((Policka[i, j] > 0) && (((i + 1) > hrana) || ((j + 1) > hrana)))
                    {
                        if (i > j)
                            hrana = i + 1;
                        else
                            hrana = j + 1;
                    }
                }
        }

        /// <summary>
        /// Vytvoří novou instanci padající kostky
        /// </summary>
        /// <param name="policka"></param>
        public Kostka(int[,] policka)
        {
            Policka = ZkopirujPolicka(policka);
            nahodnyGenerator = new Random();
            GenerujSprity();
            ZjistiHranu();
            pozice = new Vector2(0, 0);
        }

        /// <summary>
        /// Posune kostku o jedno políčko dolů
        /// </summary>
        public void Spadni()
        {
            pozice.Y++;
        }

        /// <summary>
        /// Orotuje kostku
        /// </summary>
        public void Orotuj()
        {
            // pomocné pole
            int[,] a = ZkopirujPolicka(Policka);
            // rotace pole jako matice prohozením souřadnic
            for (int y = 0; y < hrana; y++)
                for (int x = 0; x < hrana; x++)
                    Policka[x, y] = a[y, (hrana - 1) - x];
        }



        /// <summary>
        /// Vykreslí padající kostku
        /// </summary>
        /// <param name="okraj">Pozice hrací plochy na pozadí levelu</param>
        /// <param name="spriteBatch">SpriteBatch</param>
        /// <param name="sprity">Sprity k vykreslení</param>
        /// <param name="pozice">Vykreslovací pozice kostky</param>
        /// <param name="alfa">Alfaprůhlednost jako 0 až 1</param>
        private void _Vykresli(Vector2 okraj, LepsiSpriteBatch spriteBatch, Texture2D[] sprity, Vector2 pozice, float alfa)
        {
            for (int j = 0; j < 4; j++)
                for (int i = 0; i < 4; i++)
                    if (Policka[i, j] > 0)
                        spriteBatch.Draw(sprity[Policka[i, j] - 1],
                            new Vector2(okraj.X + (i + pozice.X) * sprity[0].Width,
                                okraj.Y + (j + pozice.Y) * sprity[0].Height), Color.White * alfa);
        }

        /// <summary>
        /// Vykreslí padající kostku
        /// </summary>
        /// <param name="levyOkraj">Levý okraj</param>
        /// <param name="spriteBatch">SpriteBatch</param>
        /// <param name="sprity">Sprity k vykreslení</param>
        public void Vykresli(Vector2 levyOkraj, LepsiSpriteBatch spriteBatch, Texture2D[] sprity)
        {
            _Vykresli(levyOkraj, spriteBatch, sprity, pozice, 1);
        }

        /// <summary>
        /// Vykreslí padající kostku jako zaměřovač
        /// </summary>
        /// <param name="levyOkraj">Levý okraj</param>
        /// <param name="spriteBatch">SpriteBatch</param>
        /// <param name="sprity">Sprity k vykreslení</param>
        /// <param name="pozice">Vykreslovací pozice kostky</param>
        /// <param name="alfa">Alfaprůhlednost jako 0 až 1</param>
        public void VykresliZamerovac(Vector2 levyOkraj, LepsiSpriteBatch spriteBatch, Texture2D[] sprity, Vector2 pozice, float alfa)
        {
            _Vykresli(levyOkraj, spriteBatch, sprity, pozice, alfa);
        }
    }
}
